package com.jeeplus.human.enrollment.enrollmentRegistration.controller;

import com.jeeplus.human.enrollment.enrollmentRegistration.domain.EnrollmentLanguageInfo;
import com.jeeplus.human.enrollment.enrollmentRegistration.service.EnrollmentLanguageInfoService;
import com.jeeplus.logging.annotation.ApiLog;
import com.jeeplus.logging.constant.enums.LogTypeEnum;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;

/**
 * @author 王强
 * @version 1.0
 */
@Slf4j
@RestController
@Api("语言能力")
@RequestMapping(value = "/languageInfo")
public class EnrollmentLanguageInfoController {

    @Resource
    private EnrollmentLanguageInfoService service;


    /**
     * 保存语言能力
     * @return
     */
    @ApiLog(value = "保存语言能力", type = LogTypeEnum.SAVE)
    @PostMapping("save")
    public ResponseEntity save(@Valid @RequestBody EnrollmentLanguageInfo info) throws Exception {
        service.saveOrUpdateInfo(info);
        return ResponseEntity.ok ("操作成功");
    }

    /**
     * 删除语言能力
     * @return
     */
    @ApiLog(value = "删除语言能力", type = LogTypeEnum.SAVE)
    @PostMapping("delete")
    public ResponseEntity<String> delete(@Valid @RequestBody EnrollmentLanguageInfo info) throws Exception {
        service.removeById(info.getId());
        return ResponseEntity.ok ("操作成功");
    }

    /**
     * 语言能力
     * @return
     */
    @ApiLog(value = "语言能力", type = LogTypeEnum.SAVE)
    @GetMapping("list")
    public ResponseEntity<List<EnrollmentLanguageInfo>> findList(EnrollmentLanguageInfo info) throws Exception {
        List<EnrollmentLanguageInfo> list = service.findByUserId(info.getUserId(),"5");
        return ResponseEntity.ok (list);
    }

    /**
     * 根据id查询语言能力
     * @return
     */
    @ApiLog(value = "根据id查询语言能力信息", type = LogTypeEnum.SAVE)
    @GetMapping("findById")
    public ResponseEntity<EnrollmentLanguageInfo> findById(EnrollmentLanguageInfo info) throws Exception {
        EnrollmentLanguageInfo res = service.findById(info.getId());
        return ResponseEntity.ok (res);
    }
}
