package com.jeeplus.human.enrollment.enrollmentRegistration.controller;

import com.jeeplus.human.enrollment.enrollmentRegistration.domain.EnrollmentLanguageInfo;
import com.jeeplus.human.enrollment.enrollmentRegistration.domain.EnrollmentRewardInfo;
import com.jeeplus.human.enrollment.enrollmentRegistration.service.EnrollmentLanguageInfoService;
import com.jeeplus.human.enrollment.enrollmentRegistration.service.EnrollmentRewardInfoService;
import com.jeeplus.logging.annotation.ApiLog;
import com.jeeplus.logging.constant.enums.LogTypeEnum;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;

/**
 * @author 王强
 * @version 1.0
 */
@Slf4j
@RestController
@Api("奖项信息")
@RequestMapping(value = "/rewardInfo")
public class EnrollmentRewardInfoController {

    @Resource
    private EnrollmentRewardInfoService service;


    /**
     * 保存奖项信息
     * @return
     */
    @ApiLog(value = "保存奖项信息", type = LogTypeEnum.SAVE)
    @PostMapping("save")
    public ResponseEntity save(@Valid @RequestBody EnrollmentRewardInfo info) throws Exception {
        service.saveOrUpdateInfo(info);
        return ResponseEntity.ok ("操作成功");
    }

    /**
     * 删除奖项信息
     * @return
     */
    @ApiLog(value = "删除奖项信息", type = LogTypeEnum.SAVE)
    @PostMapping("delete")
    public ResponseEntity<String> delete(@Valid @RequestBody EnrollmentRewardInfo info) throws Exception {
        service.removeById(info.getId());
        return ResponseEntity.ok ("操作成功");
    }

    /**
     * 奖项信息
     * @return
     */
    @ApiLog(value = "奖项信息", type = LogTypeEnum.SAVE)
    @GetMapping("list")
    public ResponseEntity<List<EnrollmentRewardInfo>> findList(EnrollmentRewardInfo info) throws Exception {
        List<EnrollmentRewardInfo> list = service.findByUserId(info.getUserId(),"5");
        return ResponseEntity.ok (list);
    }

    /**
     * 根据id查询奖项信息
     * @return
     */
    @ApiLog(value = "根据id查询奖项信息信息", type = LogTypeEnum.SAVE)
    @GetMapping("findById")
    public ResponseEntity<EnrollmentRewardInfo> findById(EnrollmentRewardInfo info) throws Exception {
        EnrollmentRewardInfo res = service.findById(info.getId());
        return ResponseEntity.ok (res);
    }
}
